from django.contrib.auth import authenticate
from rest_framework import serializers
from rest_framework.serializers import ModelSerializer
from . import models

class ThemeSerializer(ModelSerializer):
    class Meta:
        model = models.Theme
        fields = ('name')

class GenreSerializer(ModelSerializer):
    class Meta:
        model = models.Genre
        fields = ('name')


class ReviewSegmentSerializer(ModelSerializer):
    class Meta:
        model = models.ReviewSegment
        fields = ('position', 'length', 'comment')

class ReviewSerializer(ModelSerializer):
    reviewsegment_set = ReviewSegmentSerializer(many=True)

    class Meta:
        model = models.Review
        fields = ('pk', 'creation_date', 'summary', 'reviewsegment_set',
                  'was_seen')


class TextFullSerializer(ModelSerializer):
    review_set = ReviewSerializer(many=True)

    class Meta:
        model = models.Text
        fields = ('pk', 'title', 'content', 'hash', 'creation_date',
                  'is_published', 'genre', 'theme', 'review_set')

class TextReadSerializer(ModelSerializer):
    class Meta:
        model = models.Text
        fields = ('pk', 'title', 'content', 'hash', 'creation_date', 'genre',
                  'theme')


class UserLoginSerializer(serializers.Serializer):
    email = serializers.CharField()
    password = serializers.CharField()

    def validate(self, data):
        user = authenticate(**data)
        if user and user.is_active:
            return user

        raise NotImplementedError
